@@ -1,5 +1,5 @@
 //
-// $Id: dnstracer.c,v 1.35 2002/06/12 04:16:04 mavetju Exp $
+// $Id: dnstracer.c,v 1.37 2002/07/30 11:26:04 mavetju Exp $
 //
 
 //
@@ -318,7 +318,7 @@ void extract_data(struct dnssession *session) {
     session->recv_pkt_question=pbuffer;
     pquestion=pbuffer;
     question=(struct dnsquestion *)calloc(1,sizeof(struct dnsquestion));
-    question->query=getname(session,&pquestion);
+    question->query=strdup(getname(session,&pquestion));
     question->type=getshort(pquestion);
     question->class=getshort(pquestion+2);
     pbuffer=pquestion+4;
@@ -513,6 +513,21 @@ char *get_resource(int type,struct dnssession *session,char *buffer,int dots) {
 	strcpy(retval,printablename(getname(session,&buffer),dots));
 	return retval;
 
+    case ns_t_soa:
+    {
+	static char retval[3*NS_MAXDNAME];
+	char mname[NS_MAXDNAME];
+	char rname[NS_MAXDNAME];
+	unsigned short us;
+
+	strcpy(mname,printablename(getname(session,&buffer),dots));
+	strcpy(rname,printablename(getname(session,&buffer),dots));
+
+	us=getshort(buffer);
+	sprintf(retval,"serial: %hu mname: %s rname: %s",us,mname,rname);
+	return retval;
+    }
+	
     default: return "unknown";
     }
 
@@ -544,6 +559,8 @@ void dump_header(struct dnsheader *header) {
     case 2: printf("(Server status request)\n");break;
     case 4: printf("(Notify)\n");break;
     case 5: printf("(Update)\n");break;
+    case 14: printf("(Zone init)\n");break;
+    case 15: printf("(Zone Ref)\n");break;
     default: printf("(unknown)\n");
     }
 
@@ -1308,6 +1325,7 @@ int main(int argc,char **argv) {
 		compare("mx",	ns_t_mx		);
 		compare("ns",	ns_t_ns		);
 		compare("txt",	ns_t_txt	);
+		compare("soa",	ns_t_soa	);
 
 		if (global_querytype<1) {
 		    fprintf(stderr,
